@@ -11,9 +11,12 @@ import (
 	"strconv"
 	"strings"
 
-	"github.com/containers/podman/v3/pkg/cgroups"
+	"github.com/containers/podman/v3/pkg/rootless"
 	"github.com/cri-o/cri-o/internal/config/node"
 	libctr "github.com/opencontainers/runc/libcontainer/cgroups"
+	"github.com/opencontainers/runc/libcontainer/cgroups/fs"
+	"github.com/opencontainers/runc/libcontainer/cgroups/fs2"
+	cgcfgs "github.com/opencontainers/runc/libcontainer/configs"
 	rspec "github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
@@ -75,6 +78,9 @@ type CgroupManager interface {
 	// CreateSandboxCgroup takes the sandbox parent, and sandbox ID.
 	// It creates a new cgroup for that sandbox, which is useful when spoofing an infra container.
 	CreateSandboxCgroup(sbParent, containerID string) error
+	// RemoveSandboxCgroup takes the sandbox parent, and sandbox ID.
+	// It removes the cgroup for that sandbox, which is useful when spoofing an infra container.
+	RemoveSandboxCgroup(sbParent, containerID string) error
 }
 
 // New creates a new CgroupManager with defaults
@@ -141,17 +147,6 @@ func VerifyMemoryIsEnough(memoryLimit int64) error {
 	return nil
 }
 
-// createSandboxCgroup takes the sandbox parent, and sandbox ID.
-// It creates a new cgroup for that sandbox, which is useful when spoofing an infra container.
-func createSandboxCgroup(sbParent, containerID string, mgr CgroupManager) error {
-	path, err := mgr.ContainerCgroupAbsolutePath(sbParent, containerID)
-	if err != nil {
-		return err
-	}
-	_, err = cgroups.New(path, &rspec.LinuxResources{})
-	return err
-}
-
 // MoveProcessToContainerCgroup moves process to the container cgroup
 func MoveProcessToContainerCgroup(containerPid, commandPid int) error {
 	parentCgroupFile := fmt.Sprintf("/proc/%d/cgroup", containerPid)
@@ -173,3 +168,39 @@ func MoveProcessToContainerCgroup(containerPid, commandPid int) error {
 	}
 	return nil
 }
+
+// createSandboxCgroup takes the path of the sandbox parent and the desired containerCgroup
+// It creates a cgroup through cgroupfs (as opposed to systemd) at the location cgroupRoot/sbParent/containerCgroup.
+func createSandboxCgroup(sbParent, containerCgroup string) error {
+	mgr, err := libctrCgroupManager(sbParent, containerCgroup)
+	if err != nil {
+		return err
+	}
+	return mgr.Apply(-1)
+}
+
+func removeSandboxCgroup(sbParent, containerCgroup string) error {
+	mgr, err := libctrCgroupManager(sbParent, containerCgroup)
+	if err != nil {
+		return err
+	}
+	return mgr.Destroy()
+}
+
+func libctrCgroupManager(sbParent, containerCgroup string) (libctr.Manager, error) {
+	cg := &cgcfgs.Cgroup{
+		Name:   containerCgroup,
+		Parent: sbParent,
+		Resources: &cgcfgs.Resources{
+			SkipDevices: true,
+		},
+	}
+	if node.CgroupIsV2() {
+		return fs2.NewManager(cg, "", rootless.IsRootless())
+	}
+	return fs.NewManager(cg, nil, rootless.IsRootless()), nil
+}
+
+func containerCgroupPath(id string) string {
+	return crioPrefix + "-" + id
+}
