@@ -12,9 +12,10 @@ import (
 	"syscall"
 	"time"
 
+	"github.com/containers/common/pkg/signal"
 	"github.com/containers/podman/v3/pkg/cgroups"
-	"github.com/containers/podman/v3/pkg/signal"
 	"github.com/containers/storage/pkg/idtools"
+	"github.com/cri-o/cri-o/internal/config/nsmgr"
 	ann "github.com/cri-o/cri-o/pkg/annotations"
 	json "github.com/json-iterator/go"
 	specs "github.com/opencontainers/runtime-spec/specs-go"
@@ -22,7 +23,8 @@ import (
 	"github.com/sirupsen/logrus"
 	"golang.org/x/sys/unix"
 	"k8s.io/apimachinery/pkg/fields"
-	"k8s.io/kubernetes/pkg/kubelet/types"
+	types "k8s.io/cri-api/pkg/apis/runtime/v1"
+	kubeletTypes "k8s.io/kubernetes/pkg/kubelet/types"
 )
 
 const (
@@ -42,12 +44,10 @@ var (
 
 // Container represents a runtime container.
 type Container struct {
+	criContainer   *types.Container
 	volumes        []ContainerVolume
-	id             string
 	name           string
 	logPath        string
-	image          string
-	sandbox        string
 	runtimeHandler string
 	// this is the /var/run/storage/... directory, erased on reboot
 	bundlePath string
@@ -55,15 +55,11 @@ type Container struct {
 	dir                string
 	stopSignal         string
 	imageName          string
-	imageRef           string
 	mountPoint         string
 	seccompProfilePath string
 	conmonCgroupfsPath string
-	labels             fields.Set
-	annotations        fields.Set
 	crioAnnotations    fields.Set
 	state              *ContainerState
-	metadata           *Metadata
 	opLock             sync.RWMutex
 	spec               *specs.Spec
 	idMappings         *idtools.IDMappings
@@ -77,24 +73,25 @@ type Container struct {
 	stoppedChan        chan struct{}
 	stopStoppingChan   chan struct{}
 	stopLock           sync.Mutex
+	pidns              nsmgr.Namespace
 }
 
-// Metadata holds all necessary information for building the container name.
-// The container runtime is encouraged to expose the metadata in its user
-// interface for better user experience.
-type Metadata struct {
-	// Name of the container.
-	Name string `json:"name,omitempty"`
-
-	// Attempt number of creating the container.
-	Attempt uint32 `json:"attempt,omitempty"`
+func (c *Container) CRIAttributes() *types.ContainerAttributes {
+	return &types.ContainerAttributes{
+		Id:          c.ID(),
+		Metadata:    c.Metadata(),
+		Labels:      c.Labels(),
+		Annotations: c.Annotations(),
+	}
 }
 
 // ContainerVolume is a bind mount for the container.
 type ContainerVolume struct {
-	ContainerPath string `json:"container_path"`
-	HostPath      string `json:"host_path"`
-	Readonly      bool   `json:"readonly"`
+	ContainerPath  string                 `json:"container_path"`
+	HostPath       string                 `json:"host_path"`
+	Readonly       bool                   `json:"readonly"`
+	Propagation    types.MountPropagation `json:"propagation"`
+	SelinuxRelabel bool                   `json:"selinux_relabel"`
 }
 
 // ContainerState represents the status of a container.
@@ -114,26 +111,31 @@ type ContainerState struct {
 }
 
 // NewContainer creates a container object.
-func NewContainer(id, name, bundlePath, logPath string, labels, crioAnnotations, annotations map[string]string, image, imageName, imageRef string, metadata *Metadata, sandbox string, terminal, stdin, stdinOnce bool, runtimeHandler, dir string, created time.Time, stopSignal string) (*Container, error) {
+func NewContainer(id, name, bundlePath, logPath string, labels, crioAnnotations, annotations map[string]string, image, imageName, imageRef string, metadata *types.ContainerMetadata, sandbox string, terminal, stdin, stdinOnce bool, runtimeHandler, dir string, created time.Time, stopSignal string) (*Container, error) {
 	state := &ContainerState{}
 	state.Created = created
 	c := &Container{
-		id:               id,
+		criContainer: &types.Container{
+			Id:           id,
+			PodSandboxId: sandbox,
+			CreatedAt:    created.UnixNano(),
+			Labels:       labels,
+			Metadata:     metadata,
+			Annotations:  annotations,
+			Image: &types.ImageSpec{
+				Image: image,
+			},
+			ImageRef: imageRef,
+		},
 		name:             name,
 		bundlePath:       bundlePath,
 		logPath:          logPath,
-		labels:           labels,
-		sandbox:          sandbox,
 		terminal:         terminal,
 		stdin:            stdin,
 		stdinOnce:        stdinOnce,
 		runtimeHandler:   runtimeHandler,
-		metadata:         metadata,
-		annotations:      annotations,
 		crioAnnotations:  crioAnnotations,
-		image:            image,
 		imageName:        imageName,
-		imageRef:         imageRef,
 		dir:              dir,
 		state:            state,
 		stopSignal:       stopSignal,
@@ -149,20 +151,32 @@ func NewSpoofedContainer(id, name string, labels map[string]string, sandbox stri
 	state.Created = created
 	state.Started = created
 	c := &Container{
-		id:      id,
+		criContainer: &types.Container{
+			Id:           id,
+			CreatedAt:    created.UnixNano(),
+			Labels:       labels,
+			PodSandboxId: sandbox,
+			Metadata:     &types.ContainerMetadata{},
+			Annotations: map[string]string{
+				ann.SpoofedContainer: "true",
+			},
+			Image: &types.ImageSpec{},
+		},
 		name:    name,
-		labels:  labels,
 		spoofed: true,
 		state:   state,
 		dir:     dir,
-		sandbox: sandbox,
-	}
-	c.annotations = map[string]string{
-		ann.SpoofedContainer: "true",
 	}
 	return c
 }
 
+func (c *Container) CRIContainer() *types.Container {
+	// Return a deep copy so the State field doesn't get mutated mid-request,
+	// causing a proto panic.
+	cpy := *c.criContainer
+	return &cpy
+}
+
 // SetSpec loads the OCI spec in the container struct
 func (c *Container) SetSpec(s *specs.Spec) {
 	c.spec = s
@@ -227,7 +241,7 @@ func (c *Container) FromDisk() error {
 		if err := tmpState.SetInitPid(tmpState.Pid); err != nil {
 			return err
 		}
-		logrus.Infof("PID information for container %s updated to %d %s", c.id, tmpState.InitPid, tmpState.InitStartTime)
+		logrus.Infof("PID information for container %s updated to %d %s", c.ID(), tmpState.InitPid, tmpState.InitStartTime)
 	}
 	c.state = tmpState
 	return nil
@@ -266,7 +280,7 @@ func (c *Container) Name() string {
 
 // ID returns the id of the container.
 func (c *Container) ID() string {
-	return c.id
+	return c.criContainer.Id
 }
 
 // CleanupConmonCgroup cleans up conmon's group when using cgroupfs.
@@ -280,11 +294,11 @@ func (c *Container) CleanupConmonCgroup() {
 	}
 	cg, err := cgroups.Load(path)
 	if err != nil {
-		logrus.Infof("error loading conmon cgroup of container %s: %v", c.ID(), err)
+		logrus.Infof("Error loading conmon cgroup of container %s: %v", c.ID(), err)
 		return
 	}
 	if err := cg.Delete(); err != nil {
-		logrus.Infof("error deleting conmon cgroup of container %s: %v", c.ID(), err)
+		logrus.Infof("Error deleting conmon cgroup of container %s: %v", c.ID(), err)
 	}
 }
 
@@ -310,12 +324,12 @@ func (c *Container) LogPath() string {
 
 // Labels returns the labels of the container.
 func (c *Container) Labels() map[string]string {
-	return c.labels
+	return c.criContainer.Labels
 }
 
 // Annotations returns the annotations of the container.
 func (c *Container) Annotations() map[string]string {
-	return c.annotations
+	return c.criContainer.Annotations
 }
 
 // CrioAnnotations returns the crio annotations of the container.
@@ -325,7 +339,7 @@ func (c *Container) CrioAnnotations() map[string]string {
 
 // Image returns the image of the container.
 func (c *Container) Image() string {
-	return c.image
+	return c.criContainer.Image.Image
 }
 
 // ImageName returns the image name of the container.
@@ -335,12 +349,12 @@ func (c *Container) ImageName() string {
 
 // ImageRef returns the image ref of the container.
 func (c *Container) ImageRef() string {
-	return c.imageRef
+	return c.criContainer.ImageRef
 }
 
 // Sandbox returns the sandbox name of the container.
 func (c *Container) Sandbox() string {
-	return c.sandbox
+	return c.criContainer.PodSandboxId
 }
 
 // Dir returns the dir of the container
@@ -349,8 +363,8 @@ func (c *Container) Dir() string {
 }
 
 // Metadata returns the metadata of the container.
-func (c *Container) Metadata() *Metadata {
-	return c.metadata
+func (c *Container) Metadata() *types.ContainerMetadata {
+	return c.criContainer.Metadata
 }
 
 // State returns the state of the running container
@@ -418,7 +432,7 @@ func (c *Container) SetStartFailed(err error) {
 
 // Description returns a description for the container
 func (c *Container) Description() string {
-	return fmt.Sprintf("%s/%s/%s", c.Labels()[types.KubernetesPodNamespaceLabel], c.Labels()[types.KubernetesPodNameLabel], c.Labels()[types.KubernetesContainerNameLabel])
+	return fmt.Sprintf("%s/%s/%s", c.Labels()[kubeletTypes.KubernetesPodNamespaceLabel], c.Labels()[kubeletTypes.KubernetesPodNameLabel], c.Labels()[kubeletTypes.KubernetesContainerNameLabel])
 }
 
 // StdinOnce returns whether stdin once is set for the container.
@@ -434,7 +448,7 @@ func (c *Container) exitFilePath() string {
 // It is used to check a container state when we don't want a `$runtime state` call
 func (c *Container) IsAlive() error {
 	_, err := c.pid()
-	return errors.Wrapf(err, "checking if PID of %s is running failed", c.id)
+	return errors.Wrapf(err, "checking if PID of %s is running failed", c.ID())
 }
 
 // Pid returns the container's init PID.
@@ -586,3 +600,31 @@ func (c *Container) SetAsNotStopping() {
 	c.stopping = false
 	c.stopLock.Unlock()
 }
+
+func (c *Container) AddManagedPIDNamespace(ns nsmgr.Namespace) {
+	c.pidns = ns
+}
+
+func (c *Container) RemoveManagedPIDNamespace() error {
+	if c.pidns == nil {
+		return nil
+	}
+	return errors.Wrapf(c.pidns.Remove(), "remove PID namespace for container %s", c.ID())
+}
+
+func (c *Container) IsInfra() bool {
+	return c.ID() == c.Sandbox()
+}
+
+// nodeLevelPIDNamespace searches through the container spec to see if there is
+// a PID namespace specified. If not, it returns `true` (because the runtime spec
+// defines a node level namespace as being absent from the Namespaces list)
+func (c *Container) nodeLevelPIDNamespace() bool {
+	for i := range c.spec.Linux.Namespaces {
+		// If it's specified in the namespace list, then it is something other than Node level
+		if c.spec.Linux.Namespaces[i].Type == specs.PIDNamespace {
+			return false
+		}
+	}
+	return true
+}
