@@ -1,9 +1,7 @@
 package version
 
 import (
-	"bufio"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"reflect"
@@ -21,7 +19,7 @@ import (
 )
 
 // Version is the version of the build.
-const Version = "1.24.1"
+const Version = "1.24.6"
 
 // Variables injected during build-time
 var (
@@ -56,15 +54,13 @@ func ShouldCrioWipe(versionFileName string) (bool, error) {
 
 // shouldCrioWipe is an internal function for testing purposes
 func shouldCrioWipe(versionFileName, versionString string) (bool, error) {
-	f, err := os.Open(versionFileName)
+	if versionFileName == "" {
+		return false, nil
+	}
+	versionBytes, err := os.ReadFile(versionFileName)
 	if err != nil {
 		return true, errors.Errorf("version file %s not found: %v", versionFileName, err)
 	}
-	r := bufio.NewReader(f)
-	versionBytes, err := ioutil.ReadAll(r)
-	if err != nil {
-		return true, errors.Errorf("reading version file %s failed: %v", versionFileName, err)
-	}
 
 	// parse the version that was laid down by a previous invocation of crio
 	var oldVersion semver.Version
@@ -99,6 +95,9 @@ func LogVersion() {
 
 // writeVersionFile is an internal function for testing purposes
 func writeVersionFile(file, gitCommit, version string) error {
+	if file == "" {
+		return nil
+	}
 	current, err := parseVersionConstant(version, gitCommit)
 	// Sanity check-this should never happen
 	if err != nil {
