@@ -19,9 +19,10 @@ import (
 )
 
 const (
-	branch        = "gh-pages"
-	tokenKey      = "GITHUB_TOKEN"
-	defaultBranch = "main"
+	branch           = "gh-pages"
+	currentBranchKey = "CURRENT_BRANCH"
+	tokenKey         = "GITHUB_TOKEN"
+	defaultBranch    = "main"
 )
 
 var outputPath string
@@ -43,10 +44,10 @@ func main() {
 }
 
 func run() error {
-	// Precheck environemt
+	// Precheck environment.
 	token, tokenSet := os.LookupEnv(tokenKey)
 	if !tokenSet || token == "" {
-		logrus.Infof("%s environemt variable is not set", tokenKey)
+		logrus.Infof("%s environment variable is not set", tokenKey)
 		os.Exit(0)
 	}
 
@@ -74,16 +75,15 @@ func run() error {
 	}
 	logrus.Infof("Using HEAD commit %s", head)
 
-	targetBranch := defaultBranch
-	currentBranch, err := repo.CurrentBranch()
-	if err != nil {
-		return errors.Wrap(err, "get current branch")
+	currentBranch, currentBranchSet := os.LookupEnv(currentBranchKey)
+	if !currentBranchSet || currentBranch == "" {
+		logrus.Infof(
+			"%s environment variable is not set, using default branch `%s`",
+			currentBranchKey, defaultBranch,
+		)
+		currentBranch = defaultBranch
 	}
-	logrus.Infof("Found current branch %s", currentBranch)
-	if git.IsReleaseBranch(currentBranch) && currentBranch != defaultBranch {
-		targetBranch = currentBranch
-	}
-	logrus.Infof("Using target branch %s", targetBranch)
+	logrus.Infof("Using branch: %s", currentBranch)
 
 	templateFile, err := ioutil.TempFile("", "")
 	if err != nil {
@@ -96,12 +96,15 @@ func run() error {
 	shortHead := head[:7]
 	endRev := head
 	if output, err := command.New(
-		"git", "describe", "--exact-match",
+		"git", "describe", "--tags", "--exact-match",
 	).RunSilentSuccessOutput(); err == nil {
 		foundTag := output.OutputTrimNL()
+		logrus.Infof("Using tag via `git describe`: %s", foundTag)
 		bundleVersion = foundTag
 		shortHead = foundTag
 		endRev = foundTag
+	} else {
+		logrus.Infof("Not using git tag because `git describe` failed: %v", err)
 	}
 
 	if _, err := templateFile.WriteString(fmt.Sprintf(`# CRI-O %s
@@ -114,7 +117,9 @@ The release notes have been generated for the commit range
 Download one of our static release bundles via our Google Cloud Bucket:
 
 - [cri-o.amd64.%s.tar.gz](https://storage.googleapis.com/cri-o/artifacts/cri-o.amd64.%s.tar.gz)
+- [cri-o.amd64.%s.tar.gz.sha256sum](https://storage.googleapis.com/cri-o/artifacts/cri-o.amd64.%s.tar.gz.sha256sum)
 - [cri-o.arm64.%s.tar.gz](https://storage.googleapis.com/cri-o/artifacts/cri-o.arm64.%s.tar.gz)
+- [cri-o.arm64.%s.tar.gz.sha256sum](https://storage.googleapis.com/cri-o/artifacts/cri-o.arm64.%s.tar.gz.sha256sum)
 
 ## Changelog since %s
 
@@ -138,6 +143,8 @@ Download one of our static release bundles via our Google Cloud Bucket:
 		time.Now().Format(time.RFC1123),
 		bundleVersion, bundleVersion,
 		bundleVersion, bundleVersion,
+		bundleVersion, bundleVersion,
+		bundleVersion, bundleVersion,
 		startTag,
 	)); err != nil {
 		return errors.Wrap(err, "writing tmplate to file")
@@ -151,7 +158,7 @@ Download one of our static release bundles via our Google Cloud Bucket:
 		"./build/bin/release-notes",
 		"--org=cri-o",
 		"--repo=cri-o",
-		"--branch="+targetBranch,
+		"--branch="+currentBranch,
 		"--repo-path=/tmp/cri-o-repo",
 		"--required-author=",
 		"--start-rev="+startTag,
