@@ -6,18 +6,18 @@ import (
 	"github.com/cri-o/cri-o/internal/log"
 	oci "github.com/cri-o/cri-o/internal/oci"
 	"github.com/cri-o/cri-o/internal/runtimehandlerhooks"
-	"github.com/cri-o/cri-o/server/cri/types"
 	"golang.org/x/net/context"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
+	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 )
 
 // StartContainer starts the container.
 func (s *Server) StartContainer(ctx context.Context, req *types.StartContainerRequest) (retErr error) {
-	log.Infof(ctx, "Starting container: %s", req.ContainerID)
-	c, err := s.GetContainerFromShortID(req.ContainerID)
+	log.Infof(ctx, "Starting container: %s", req.ContainerId)
+	c, err := s.GetContainerFromShortID(req.ContainerId)
 	if err != nil {
-		return status.Errorf(codes.NotFound, "could not find container %q: %v", req.ContainerID, err)
+		return status.Errorf(codes.NotFound, "could not find container %q: %v", req.ContainerId, err)
 	}
 	state := c.State()
 	if state.Status != oci.ContainerStateCreated {
@@ -39,12 +39,12 @@ func (s *Server) StartContainer(ctx context.Context, req *types.StartContainerRe
 			c.SetStartFailed(retErr)
 			if hooks != nil {
 				if err := hooks.PreStop(ctx, c, sandbox); err != nil {
-					log.Warnf(ctx, "failed to run pre-stop hook for container %q: %v", c.ID(), err)
+					log.Warnf(ctx, "Failed to run pre-stop hook for container %q: %v", c.ID(), err)
 				}
 			}
 		}
 		if err := s.ContainerStateToDisk(ctx, c); err != nil {
-			log.Warnf(ctx, "unable to write containers %s state to disk: %v", c.ID(), err)
+			log.Warnf(ctx, "Unable to write containers %s state to disk: %v", c.ID(), err)
 		}
 	}()
 
@@ -58,6 +58,11 @@ func (s *Server) StartContainer(ctx context.Context, req *types.StartContainerRe
 		return fmt.Errorf("failed to start container %s: %v", c.ID(), err)
 	}
 
-	log.Infof(ctx, "Started container %s: %s", c.ID(), c.Description())
+	log.WithFields(ctx, map[string]interface{}{
+		"description": c.Description(),
+		"containerID": c.ID(),
+		"sandboxID":   sandbox.ID(),
+		"PID":         state.Pid,
+	}).Infof("Started container")
 	return nil
 }
