@@ -6,8 +6,8 @@ import (
 	pb "k8s.io/cri-api/pkg/apis/runtime/v1"
 )
 
-func (c *service) ContainerStatus(
+func (s *service) ContainerStatus(
 	ctx context.Context, req *pb.ContainerStatusRequest,
 ) (*pb.ContainerStatusResponse, error) {
-	return nil, nil
+	return s.server.ContainerStatus(ctx, req)
 }
