@@ -2,22 +2,16 @@ package v1alpha2
 
 import (
 	"context"
+	"unsafe"
 
-	"github.com/cri-o/cri-o/server/cri/types"
+	v1 "k8s.io/cri-api/pkg/apis/runtime/v1"
 	pb "k8s.io/cri-api/pkg/apis/runtime/v1alpha2"
 )
 
 func (s *service) RemoveImage(
 	ctx context.Context, req *pb.RemoveImageRequest,
 ) (*pb.RemoveImageResponse, error) {
-	r := &types.RemoveImageRequest{}
-	if req.Image != nil {
-		r.Image = &types.ImageSpec{
-			Image:       req.Image.Image,
-			Annotations: req.Image.Annotations,
-		}
-	}
-	if err := s.server.RemoveImage(ctx, r); err != nil {
+	if err := s.server.RemoveImage(ctx, (*v1.RemoveImageRequest)(unsafe.Pointer(req))); err != nil {
 		return nil, err
 	}
 	return &pb.RemoveImageResponse{}, nil
