@@ -4,6 +4,7 @@ import (
 	"fmt"
 	"math"
 	"net/http"
+	"net/http/pprof"
 
 	"github.com/containers/storage/pkg/idtools"
 	"github.com/cri-o/cri-o/internal/lib/sandbox"
@@ -55,11 +56,13 @@ var (
 
 func (s *Server) getContainerInfo(id string, getContainerFunc, getInfraContainerFunc func(id string) *oci.Container, getSandboxFunc func(id string) *sandbox.Sandbox) (types.ContainerInfo, error) {
 	ctr := getContainerFunc(id)
+	isInfra := false
 	if ctr == nil {
 		ctr = getInfraContainerFunc(id)
 		if ctr == nil {
 			return types.ContainerInfo{}, errCtrNotFound
 		}
+		isInfra = true
 	}
 	// TODO(mrunalp): should we call UpdateStatus()?
 	ctrState := ctr.State()
@@ -68,19 +71,30 @@ func (s *Server) getContainerInfo(id string, getContainerFunc, getInfraContainer
 	}
 	sb := getSandboxFunc(ctr.Sandbox())
 	if sb == nil {
-		logrus.Debugf("can't find sandbox %s for container %s", ctr.Sandbox(), id)
+		logrus.Debugf("Can't find sandbox %s for container %s", ctr.Sandbox(), id)
 		return types.ContainerInfo{}, errSandboxNotFound
 	}
-	image := ctr.Image()
-	if s.ContainerServer != nil && s.ContainerServer.StorageImageServer() != nil {
-		if status, err := s.ContainerServer.StorageImageServer().ImageStatus(s.config.SystemContext, ctr.ImageRef()); err == nil {
-			image = status.Name
+
+	pidToReturn := ctrState.Pid
+	if isInfra && pidToReturn == 0 {
+		// It is possible the infra container doesn't report a PID.
+		// That can either happen if we're using a vm based runtime,
+		// or if we've dropped the infra container.
+		// Since the Pid is used exclusively to find the network stats,
+		// and pods share their network (whether it's host or pod level)
+		// we can return the pid of a running container in the pod.
+		for _, c := range sb.Containers().List() {
+			ctrPid, err := c.Pid()
+			if ctrPid > 0 && err == nil {
+				pidToReturn = ctrPid
+				break
+			}
 		}
 	}
 	return types.ContainerInfo{
 		Name:            ctr.Name(),
-		Pid:             ctrState.Pid,
-		Image:           image,
+		Pid:             pidToReturn,
+		Image:           ctr.ImageName(),
 		ImageRef:        ctr.ImageRef(),
 		CreatedTime:     ctrState.Created.UnixNano(),
 		Labels:          ctr.Labels(),
@@ -97,10 +111,12 @@ const (
 	InspectConfigEndpoint     = "/config"
 	InspectContainersEndpoint = "/containers"
 	InspectInfoEndpoint       = "/info"
+	InspectPauseEndpoint      = "/pause"
+	InspectUnpauseEndpoint    = "/unpause"
 )
 
-// GetInfoMux returns the mux used to serve info requests
-func (s *Server) GetInfoMux() *bone.Mux {
+// GetExtendInterfaceMux returns the mux used to serve extend interface requests
+func (s *Server) GetExtendInterfaceMux(enableProfile bool) *bone.Mux {
 	mux := bone.New()
 
 	mux.Get(InspectConfigEndpoint, http.HandlerFunc(func(w http.ResponseWriter, req *http.Request) {
@@ -112,8 +128,7 @@ func (s *Server) GetInfoMux() *bone.Mux {
 
 		w.Header().Set("Content-Type", "application/toml")
 		if _, err := w.Write(b); err != nil {
-			http.Error(w, fmt.Sprintf("unable to write TOML: %v", err),
-				http.StatusInternalServerError)
+			logrus.Errorf("Unable to write response TOML: %v", err)
 		}
 	}))
 
@@ -126,7 +141,7 @@ func (s *Server) GetInfoMux() *bone.Mux {
 		}
 		w.Header().Set("Content-Type", "application/json")
 		if _, err := w.Write(js); err != nil {
-			http.Error(w, fmt.Sprintf("unable to write JSON: %v", err), http.StatusInternalServerError)
+			logrus.Errorf("Unable to write response JSON: %v", err)
 		}
 	}))
 
@@ -153,9 +168,76 @@ func (s *Server) GetInfoMux() *bone.Mux {
 		}
 		w.Header().Set("Content-Type", "application/json")
 		if _, err := w.Write(js); err != nil {
-			http.Error(w, fmt.Sprintf("unable to write JSON: %v", err), http.StatusInternalServerError)
+			logrus.Errorf("Unable to write response JSON: %v", err)
 		}
 	}))
 
+	mux.Get(InspectPauseEndpoint+"/:id", http.HandlerFunc(func(w http.ResponseWriter, req *http.Request) {
+		containerID := bone.GetValue(req, "id")
+		ctr := s.GetContainer(containerID)
+
+		if ctr == nil {
+			http.Error(w, fmt.Sprintf("can't find the container with id %s", containerID), http.StatusNotFound)
+			return
+		}
+		ctrStatus := ctr.State().Status
+		if ctrStatus != oci.ContainerStateRunning && ctrStatus != oci.ContainerStateCreated {
+			http.Error(w,
+				fmt.Sprintf("container is not in running or created state, now is %s", ctrStatus),
+				http.StatusConflict)
+			return
+		}
+		if err := s.Runtime().PauseContainer(s.stream.ctx, ctr); err != nil {
+			http.Error(w, err.Error(), http.StatusInternalServerError)
+			return
+		}
+		if err := s.Runtime().UpdateContainerStatus(s.stream.ctx, ctr); err != nil {
+			http.Error(w, err.Error(), http.StatusInternalServerError)
+			return
+		}
+		w.Header().Set("Content-Type", "text/html")
+		if _, err := w.Write([]byte("200 OK")); err != nil {
+			logrus.Errorf("Unable to write response JSON: %v", err)
+		}
+	}))
+
+	mux.Get(InspectUnpauseEndpoint+"/:id", http.HandlerFunc(func(w http.ResponseWriter, req *http.Request) {
+		containerID := bone.GetValue(req, "id")
+		ctr := s.GetContainer(containerID)
+
+		if ctr == nil {
+			http.Error(w, fmt.Sprintf("can't find the container with id %s", containerID), http.StatusNotFound)
+			return
+		}
+		ctrStatus := ctr.State().Status
+		if ctrStatus != oci.ContainerStatePaused {
+			http.Error(w,
+				fmt.Sprintf("container is not in paused state, now is %s", ctrStatus),
+				http.StatusConflict)
+			return
+		}
+		if err := s.Runtime().UnpauseContainer(s.stream.ctx, ctr); err != nil {
+			http.Error(w, err.Error(), http.StatusInternalServerError)
+			return
+		}
+		if err := s.Runtime().UpdateContainerStatus(s.stream.ctx, ctr); err != nil {
+			http.Error(w, err.Error(), http.StatusInternalServerError)
+			return
+		}
+		w.Header().Set("Content-Type", "text/html")
+		if _, err := w.Write([]byte("200 OK")); err != nil {
+			logrus.Errorf("Unable to write response JSON: %v", err)
+		}
+	}))
+
+	// Add pprof handlers
+	if enableProfile {
+		mux.Get("/debug/pprof/cmdline", http.HandlerFunc(pprof.Cmdline))
+		mux.Get("/debug/pprof/profile", http.HandlerFunc(pprof.Profile))
+		mux.Get("/debug/pprof/symbol", http.HandlerFunc(pprof.Symbol))
+		mux.Get("/debug/pprof/trace", http.HandlerFunc(pprof.Trace))
+		mux.Get("/debug/pprof/*", http.HandlerFunc(pprof.Index))
+	}
+
 	return mux
 }
