@@ -4,10 +4,10 @@ import (
 	"context"
 
 	"github.com/cri-o/cri-o/internal/oci"
-	"github.com/cri-o/cri-o/server/cri/types"
-	. "github.com/onsi/ginkgo"
+	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	specs "github.com/opencontainers/runtime-spec/specs-go"
+	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 )
 
 // The actual test suite
@@ -81,7 +81,7 @@ var _ = t.Describe("ListPodSandbox", func() {
 			// When
 			response, err := sut.ListPodSandbox(context.Background(),
 				&types.ListPodSandboxRequest{Filter: &types.PodSandboxFilter{
-					ID: sandboxID,
+					Id: sandboxID,
 				}})
 
 			// Then
@@ -100,9 +100,9 @@ var _ = t.Describe("ListPodSandbox", func() {
 			// When
 			response, err := sut.ListPodSandbox(context.Background(),
 				&types.ListPodSandboxRequest{Filter: &types.PodSandboxFilter{
-					ID: sandboxID,
+					Id: sandboxID,
 					State: &types.PodSandboxStateValue{
-						State: types.PodSandboxStateSandboxReady,
+						State: types.PodSandboxState_SANDBOX_READY,
 					},
 				}})
 
@@ -122,7 +122,7 @@ var _ = t.Describe("ListPodSandbox", func() {
 			// When
 			response, err := sut.ListPodSandbox(context.Background(),
 				&types.ListPodSandboxRequest{Filter: &types.PodSandboxFilter{
-					ID:            sandboxID,
+					Id:            sandboxID,
 					LabelSelector: map[string]string{"label": "value"},
 				}})
 
@@ -139,7 +139,7 @@ var _ = t.Describe("ListPodSandbox", func() {
 			// When
 			response, err := sut.ListPodSandbox(context.Background(),
 				&types.ListPodSandboxRequest{Filter: &types.PodSandboxFilter{
-					ID: sandboxID,
+					Id: sandboxID,
 				}})
 
 			// Then
