@@ -1,5 +1,3 @@
-// +build linux
-
 package server
 
 import (
@@ -12,7 +10,7 @@ import (
 	"time"
 
 	cnitypes "github.com/containernetworking/cni/pkg/types"
-	current "github.com/containernetworking/cni/pkg/types/current"
+	current "github.com/containernetworking/cni/pkg/types/100"
 	"github.com/containers/podman/v3/pkg/annotations"
 	"github.com/containers/podman/v3/pkg/rootless"
 	selinux "github.com/containers/podman/v3/pkg/selinux"
@@ -20,6 +18,8 @@ import (
 	"github.com/containers/storage/pkg/idtools"
 	"github.com/cri-o/cri-o/internal/config/node"
 	"github.com/cri-o/cri-o/internal/config/nsmgr"
+	ctrfactory "github.com/cri-o/cri-o/internal/factory/container"
+	sboxfactory "github.com/cri-o/cri-o/internal/factory/sandbox"
 	"github.com/cri-o/cri-o/internal/lib"
 	libsandbox "github.com/cri-o/cri-o/internal/lib/sandbox"
 	"github.com/cri-o/cri-o/internal/log"
@@ -27,8 +27,6 @@ import (
 	"github.com/cri-o/cri-o/internal/resourcestore"
 	ann "github.com/cri-o/cri-o/pkg/annotations"
 	libconfig "github.com/cri-o/cri-o/pkg/config"
-	"github.com/cri-o/cri-o/pkg/sandbox"
-	"github.com/cri-o/cri-o/server/cri/types"
 	"github.com/cri-o/cri-o/utils"
 	json "github.com/json-iterator/go"
 	spec "github.com/opencontainers/runtime-spec/specs-go"
@@ -38,6 +36,7 @@ import (
 	"golang.org/x/net/context"
 	"golang.org/x/sys/unix"
 	"k8s.io/apimachinery/pkg/api/resource"
+	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 	kubeletTypes "k8s.io/kubernetes/pkg/kubelet/types"
 )
 
@@ -91,11 +90,20 @@ func (s *Server) configureSandboxIDMappings(mode string, sc *types.LinuxSandboxS
 
 	_, uidMappingsPresent := values["uidmapping"]
 	_, gidMappingsPresent := values["gidmapping"]
-	// allow these options only if running as root
+	// limit mappings for pods that aren't going to be running as root
+	minimumMappableUID, minimumMappableGID := s.minimumMappableUID, s.minimumMappableGID
 	if uidMappingsPresent || gidMappingsPresent {
 		user := sc.RunAsUser
-		if user == nil || user.Value != 0 {
-			return nil, errors.New("cannot use uidmapping or gidmapping if not running as root")
+		if user == nil {
+			return nil, errors.New("cannot use uidmapping or gidmapping if RunAsUser is not set")
+		}
+		if user.Value != 0 {
+			if minimumMappableUID < 0 {
+				return nil, errors.New("cannot use uidmapping or gidmapping if not running as root and minimum mappable ID is not set")
+			}
+			if user.Value < minimumMappableUID {
+				return nil, errors.Errorf("cannot use uidmapping or gidmapping if running as a UID below minimum mappable ID %d", minimumMappableUID)
+			}
 		}
 	}
 
@@ -186,6 +194,21 @@ func (s *Server) configureSandboxIDMappings(mode string, sc *types.LinuxSandboxS
 				ret.AutoUserNsOpts.AdditionalGIDMappings = m
 			}
 		}
+		// make sure we haven't asked to map any sensitive and/or privileged IDs
+		if minimumMappableUID >= 0 {
+			for _, uidSlice := range ret.AutoUserNsOpts.AdditionalUIDMappings {
+				if int64(uidSlice.HostID) < minimumMappableUID {
+					return nil, errors.Errorf("not allowed to map UID range (%d-%d), below minimum mappable UID %d", uidSlice.HostID, uidSlice.HostID+uidSlice.Size-1, minimumMappableUID)
+				}
+			}
+		}
+		if minimumMappableGID >= 0 {
+			for _, gidSlice := range ret.AutoUserNsOpts.AdditionalGIDMappings {
+				if int64(gidSlice.HostID) < minimumMappableGID {
+					return nil, errors.Errorf("not allowed to map GID range (%d-%d), below minimum mappable GID %d", gidSlice.HostID, gidSlice.HostID+gidSlice.Size-1, minimumMappableGID)
+				}
+			}
+		}
 		return ret, nil
 	case "private":
 		var err error
@@ -233,6 +256,22 @@ func (s *Server) configureSandboxIDMappings(mode string, sc *types.LinuxSandboxS
 			gids = addToMappingsIfMissing(gids, g)
 		}
 
+		// make sure we haven't mapped any sensitive and/or privileged IDs
+		if minimumMappableUID >= 0 {
+			for _, uidSlice := range uids {
+				if int64(uidSlice.HostID) < minimumMappableUID {
+					return nil, errors.Errorf("not allowed to map UID range (%d-%d), below minimum mappable UID %d", uidSlice.HostID, uidSlice.HostID+uidSlice.Size-1, minimumMappableUID)
+				}
+			}
+		}
+		if minimumMappableGID >= 0 {
+			for _, gidSlice := range gids {
+				if int64(gidSlice.HostID) < minimumMappableGID {
+					return nil, errors.Errorf("not allowed to map GID range (%d-%d), below minimum mappable GID %d", gidSlice.HostID, gidSlice.HostID+gidSlice.Size-1, minimumMappableGID)
+				}
+			}
+		}
+
 		return &storage.IDMappingOptions{UIDMap: uids, GIDMap: gids}, nil
 	}
 	return nil, errors.Errorf("invalid userns mode: %q", mode)
@@ -269,10 +308,7 @@ func (s *Server) getSandboxIDMappings(sb *libsandbox.Sandbox) (*idtools.IDMappin
 }
 
 func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequest) (resp *types.RunPodSandboxResponse, retErr error) {
-	s.updateLock.RLock()
-	defer s.updateLock.RUnlock()
-
-	sbox := sandbox.New()
+	sbox := sboxfactory.New()
 	if err := sbox.SetConfig(req.Config); err != nil {
 		return nil, errors.Wrap(err, "setting sandbox config")
 	}
@@ -280,7 +316,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 	pathsToChown := []string{}
 
 	// we need to fill in the container name, as it is not present in the request. Luckily, it is a constant.
-	log.Infof(ctx, "Running pod sandbox: %s%s", translateLabelsToDescription(sbox.Config().Labels), types.InfraContainerName)
+	log.Infof(ctx, "Running pod sandbox: %s%s", translateLabelsToDescription(sbox.Config().Labels), oci.InfraContainerName)
 
 	kubeName := sbox.Config().Metadata.Name
 	namespace := sbox.Config().Metadata.Namespace
@@ -313,13 +349,19 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		}
 		cachedID, resourceErr := s.getResourceOrWait(ctx, sbox.Name(), "sandbox")
 		if resourceErr == nil {
-			return &types.RunPodSandboxResponse{PodSandboxID: cachedID}, nil
+			return &types.RunPodSandboxResponse{PodSandboxId: cachedID}, nil
 		}
 		return nil, errors.Wrapf(err, resourceErr.Error())
 	}
 
+	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox creating")
+
 	securityContext := sbox.Config().Linux.SecurityContext
-	hostNetwork := securityContext.NamespaceOptions.Network == types.NamespaceModeNODE
+
+	if securityContext.NamespaceOptions == nil {
+		securityContext.NamespaceOptions = &types.NamespaceOption{}
+	}
+	hostNetwork := securityContext.NamespaceOptions.Network == types.NamespaceMode_NODE
 
 	if err := s.config.CNIPluginReadyOrError(); err != nil && !hostNetwork {
 		// if the cni plugin isn't ready yet, we should wait until it is
@@ -331,6 +373,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		}
 		log.Infof(ctx, "CNI plugin is now ready. Continuing to create %s", sbox.Name())
 	}
+	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox network ready")
 
 	description := fmt.Sprintf("runSandbox: releasing pod sandbox name: %s", sbox.Name())
 	resourceCleaner.Add(ctx, description, func() error {
@@ -345,8 +388,8 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		return nil, err
 	}
 
-	if err := s.Runtime().FilterDisallowedAnnotations(runtimeHandler, sbox.Config().Annotations); err != nil {
-		return nil, errors.Wrap(err, "filter disallowed annotations")
+	if err := s.FilterDisallowedAnnotations(sbox.Config().Annotations, sbox.Config().Annotations, runtimeHandler); err != nil {
+		return nil, err
 	}
 
 	kubeAnnotations := sbox.Config().Annotations
@@ -377,6 +420,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 
 	privileged := s.privilegedSandbox(req)
 
+	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox storage creation")
 	podContainer, err := s.StorageRuntimeServer().CreatePodSandbox(s.config.SystemContext,
 		sbox.Name(), sbox.ID(),
 		s.config.PauseImage,
@@ -384,7 +428,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		"",
 		containerName,
 		kubeName,
-		sbox.Config().Metadata.UID,
+		sbox.Config().Metadata.Uid,
 		namespace,
 		attempt,
 		idMappingsOptions,
@@ -433,7 +477,6 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 	if err := sbox.InitInfraContainer(&s.config, &podContainer); err != nil {
 		return nil, err
 	}
-	g := sbox.Spec()
 	pathsToChown = append(pathsToChown, sbox.ResolvPath())
 
 	// add metadata
@@ -452,7 +495,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 
 	// Add special container name label for the infra container
 	if labels != nil {
-		labels[kubeletTypes.KubernetesContainerNameLabel] = types.InfraContainerName
+		labels[kubeletTypes.KubernetesContainerNameLabel] = oci.InfraContainerName
 	}
 	labelsJSON, err := json.Marshal(labels)
 	if err != nil {
@@ -465,25 +508,19 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		return nil, err
 	}
 
-	// Add capabilities from crio.conf if default_capabilities is defined
-	capabilities := &types.Capability{}
-	g.ClearProcessCapabilities()
-	if err := setupCapabilities(g, capabilities, s.config.DefaultCapabilities); err != nil {
-		return nil, err
-	}
-
 	nsOptsJSON, err := json.Marshal(securityContext.NamespaceOptions)
 	if err != nil {
 		return nil, err
 	}
 
-	hostIPC := securityContext.NamespaceOptions.Ipc == types.NamespaceModeNODE
-	hostPID := securityContext.NamespaceOptions.Pid == types.NamespaceModeNODE
+	hostIPC := securityContext.NamespaceOptions.Ipc == types.NamespaceMode_NODE
+	hostPID := securityContext.NamespaceOptions.Pid == types.NamespaceMode_NODE
 
 	// Don't use SELinux separation with Host Pid or IPC Namespace or privileged.
 	if hostPID || hostIPC {
 		processLabel, mountLabel = "", ""
 	}
+	g := sbox.Spec()
 	g.SetProcessSelinuxLabel(processLabel)
 	g.SetLinuxMountLabel(mountLabel)
 
@@ -491,6 +528,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 	g.RemoveMount(libsandbox.DevShmPath)
 
 	// create shm mount for the pod containers.
+	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox shm creation")
 	var shmPath string
 	if hostIPC {
 		shmPath = libsandbox.DevShmPath
@@ -519,6 +557,8 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		})
 	}
 
+	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox spec configuration")
+
 	mnt := spec.Mount{
 		Type:        "bind",
 		Source:      shmPath,
@@ -570,10 +610,12 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 	g.AddAnnotation(annotations.Annotations, string(kubeAnnotationsJSON))
 	g.AddAnnotation(annotations.LogPath, logPath)
 	g.AddAnnotation(annotations.Name, sbox.Name())
+	g.AddAnnotation(annotations.SandboxName, sbox.Name())
 	g.AddAnnotation(annotations.Namespace, namespace)
 	g.AddAnnotation(annotations.ContainerType, annotations.ContainerTypeSandbox)
 	g.AddAnnotation(annotations.SandboxID, sbox.ID())
 	g.AddAnnotation(annotations.Image, s.config.PauseImage)
+	g.AddAnnotation(annotations.ImageName, s.config.PauseImage)
 	g.AddAnnotation(annotations.ContainerName, containerName)
 	g.AddAnnotation(annotations.ContainerID, sbox.ID())
 	g.AddAnnotation(annotations.ShmPath, shmPath)
@@ -625,13 +667,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		}
 	}
 
-	sbMetadata := &libsandbox.Metadata{
-		Name:      metadata.Name,
-		UID:       metadata.UID,
-		Namespace: metadata.Namespace,
-		Attempt:   metadata.Attempt,
-	}
-	sb, err := libsandbox.New(sbox.ID(), namespace, sbox.Name(), kubeName, logDir, labels, kubeAnnotations, processLabel, mountLabel, sbMetadata, shmPath, cgroupParent, privileged, runtimeHandler, sbox.ResolvPath(), hostname, portMappings, hostNetwork, created, usernsMode)
+	sb, err := libsandbox.New(sbox.ID(), namespace, sbox.Name(), kubeName, logDir, labels, kubeAnnotations, processLabel, mountLabel, metadata, shmPath, cgroupParent, privileged, runtimeHandler, sbox.ResolvPath(), hostname, portMappings, hostNetwork, created, usernsMode)
 	if err != nil {
 		return nil, err
 	}
@@ -658,6 +694,10 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		log.Infof(ctx, description)
 		err := s.PodIDIndex().Delete(sbox.ID())
 		if err != nil {
+			// already deleted
+			if strings.Contains(err.Error(), noSuchID) {
+				return nil
+			}
 			log.Warnf(ctx, "Could not delete pod id %s from idIndex", sbox.ID())
 		}
 		return err
@@ -674,6 +714,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 	sysctls := s.configureGeneratorForSysctls(ctx, g, hostNetwork, hostIPC, req.Config.Linux.Sysctls)
 
 	// set up namespaces
+	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox namespace creation")
 	nsCleanupFuncs, err := s.configureGeneratorForSandboxNamespaces(hostNetwork, hostIPC, hostPID, sandboxIDMappings, sysctls, sb, g)
 	// We want to cleanup after ourselves if we are managing any namespaces and fail in this function.
 	nsCleanupDescription := fmt.Sprintf("runSandbox: cleaning up namespaces after failing to run sandbox %s", sbox.ID())
@@ -696,6 +737,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 	var ips []string
 	var result cnitypes.Result
 
+	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox network creation")
 	ips, result, err = s.networkStart(ctx, sb)
 	if err != nil {
 		resourceCleaner.Add(ctx, nsCleanupDescription, nsCleanupFunc)
@@ -725,6 +767,18 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		}
 		g.AddAnnotation(annotations.CNIResult, string(cniResultJSON))
 	}
+	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox storage start")
+
+	mountPoint, err := s.StorageRuntimeServer().StartContainer(sbox.ID())
+	if err != nil {
+		return nil, fmt.Errorf("failed to mount container %s in pod sandbox %s(%s): %w", containerName, sb.Name(), sbox.ID(), err)
+	}
+	resourceCleaner.Add(ctx, "runSandbox: stopping storage container for sandbox "+sbox.ID(), func() error {
+		if err := s.StorageRuntimeServer().StopContainer(sbox.ID()); err != nil {
+			return fmt.Errorf("could not stop storage container: %s: %w", sbox.ID(), err)
+		}
+		return nil
+	})
 
 	// Set OOM score adjust of the infra container to be very low
 	// so it doesn't get killed.
@@ -739,19 +793,6 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 	}
 
 	saveOptions := generate.ExportOptions{}
-	mountPoint, err := s.StorageRuntimeServer().StartContainer(sbox.ID())
-	if err != nil {
-		return nil, fmt.Errorf("failed to mount container %s in pod sandbox %s(%s): %v", containerName, sb.Name(), sbox.ID(), err)
-	}
-	description = fmt.Sprintf("runSandbox: stopping storage container for sandbox %s", sbox.ID())
-	resourceCleaner.Add(ctx, description, func() error {
-		log.Infof(ctx, description)
-		err2 := s.StorageRuntimeServer().StopContainer(sbox.ID())
-		if err2 != nil {
-			log.Warnf(ctx, "Could not stop storage container: %v: %v", sbox.ID(), err2)
-		}
-		return err2
-	})
 	g.AddAnnotation(annotations.MountPoint, mountPoint)
 
 	hostnamePath := fmt.Sprintf("%s/hostname", podContainer.RunDir)
@@ -773,7 +814,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 	sb.AddHostnamePath(hostnamePath)
 
 	if sandboxIDMappings != nil {
-		if securityContext.NamespaceOptions.Ipc == types.NamespaceModeNODE {
+		if securityContext.NamespaceOptions.Ipc == types.NamespaceMode_NODE {
 			g.RemoveMount("/dev/mqueue")
 			mqueue := spec.Mount{
 				Type:        "bind",
@@ -794,7 +835,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 			}
 			g.AddMount(sysMnt)
 		}
-		if securityContext.NamespaceOptions.Pid == types.NamespaceModeNODE {
+		if securityContext.NamespaceOptions.Pid == types.NamespaceMode_NODE {
 			g.RemoveMount("/proc")
 			proc := spec.Mount{
 				Type:        "bind",
@@ -875,6 +916,10 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		return nil, err
 	}
 
+	if err := sb.SetContainerEnvFile(); err != nil {
+		return nil, err
+	}
+
 	if err = g.SaveToFile(filepath.Join(podContainer.Dir, "config.json"), saveOptions); err != nil {
 		return nil, fmt.Errorf("failed to save template configuration for pod sandbox %s(%s): %v", sb.Name(), sbox.ID(), err)
 	}
@@ -902,6 +947,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		}
 	}
 
+	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox container runtime creation")
 	if err := s.createContainerPlatform(ctx, container, sb.CgroupParent(), sandboxIDMappings); err != nil {
 		return nil, err
 	}
@@ -950,7 +996,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 	sb.SetCreated()
 
 	log.Infof(ctx, "Ran pod sandbox %s with infra container: %s", container.ID(), container.Description())
-	resp = &types.RunPodSandboxResponse{PodSandboxID: sbox.ID()}
+	resp = &types.RunPodSandboxResponse{PodSandboxId: sbox.ID()}
 	return resp, nil
 }
 
@@ -1045,36 +1091,9 @@ func (s *Server) configureGeneratorForSandboxNamespaces(hostNetwork, hostIPC, ho
 
 	cleanupFuncs = append(cleanupFuncs, sb.RemoveManagedNamespaces)
 
-	if err := configureGeneratorGivenNamespacePaths(sb.NamespacePaths(), g); err != nil {
+	if err := ctrfactory.ConfigureGeneratorGivenNamespacePaths(sb.NamespacePaths(), g); err != nil {
 		return cleanupFuncs, err
 	}
 
 	return cleanupFuncs, nil
 }
-
-// configureGeneratorGivenNamespacePaths takes a map of nsType -> nsPath. It configures the generator
-// to add or replace the defaults to these paths
-func configureGeneratorGivenNamespacePaths(managedNamespaces []*libsandbox.ManagedNamespace, g *generate.Generator) error {
-	typeToSpec := map[nsmgr.NSType]spec.LinuxNamespaceType{
-		nsmgr.IPCNS:  spec.IPCNamespace,
-		nsmgr.NETNS:  spec.NetworkNamespace,
-		nsmgr.UTSNS:  spec.UTSNamespace,
-		nsmgr.USERNS: spec.UserNamespace,
-	}
-
-	for _, ns := range managedNamespaces {
-		// allow for empty paths, as this namespace just shouldn't be configured
-		if ns.Path() == "" {
-			continue
-		}
-		nsForSpec := typeToSpec[ns.Type()]
-		if nsForSpec == "" {
-			return errors.Errorf("Invalid namespace type %s", nsForSpec)
-		}
-		err := g.AddOrReplaceLinuxNamespace(string(nsForSpec), ns.Path())
-		if err != nil {
-			return err
-		}
-	}
-	return nil
-}
